@@ -1,3 +1,46 @@
+c Exchange two values
+	subroutine swap2(v1, v2)
+	real v1, v2
+	real temp
+
+	temp = v1
+	v1 = v2
+	v2 = temp
+
+	end
+
+c We often say what the current value of a parameter is
+c when we request a new value
+c this is supposed to save duplicated code
+
+	subroutine nowis(value)
+	include 'robcom'
+	write(tstring, *)'Current value = ',value
+	call sprompt(tstring)
+
+	end
+
+
+
+
+c set msiz from a C routine
+
+	subroutine setmsiz(invalue)
+	integer invalue
+	include 'msizcom'
+	msiz = invalue
+	end
+
+c get msiz from a C routine
+
+	subroutine getmsiz(invalue)
+	integer invalue
+	include 'msizcom'
+	invalue = msiz
+	end
+	
+
+
 c get starting position for axis labelling
 	subroutine getmin(start, min, dist)
 	real start, min, dist
@@ -236,8 +279,8 @@ c if id=2 draw at top and bottom
       subroutine erbox(npt,xpt,ypt,dxm,dxp,dym,dyp)
 
       integer*4 npt,ido
-      real*4 xpt(npt),ypt(npt),dxm(npt),dxp(npt),dym(npt),dyp(npt)
-      real*4 x,y,xlt,xrt,ybt,ytp
+      real xpt(*),ypt(*),dxm(*),dxp(*),dym(*),dyp(*)
+      real x,y,xlt,xrt,ybt,ytp
       common/limits/xminp,xmaxp,yminp,ymaxp
 	common/test/xtmin, xtmax, ytmin, ytmax
 
@@ -272,8 +315,8 @@ c if id=2 draw at top and bottom
       subroutine erboxf(npt,xpt,ypt,dxm,dxp,dym,dyp)
 c  filled error box
       integer*4 npt,ido
-      real*4 xpt(npt),ypt(npt),dxm(npt),dxp(npt),dym(npt),dyp(npt)
-      real*4 x,y,xlt,xrt,ybt,ytp
+      real xpt(*),ypt(*),dxm(*),dxp(*),dym(*),dyp(*)
+      real x,y,xlt,xrt,ybt,ytp
       common/limits/xminp,xmaxp,yminp,ymaxp
 	common/test/xtmin, xtmax, ytmin, ytmax
 
@@ -308,8 +351,8 @@ c  filled error box
       common/limits/xminp,xmaxp,yminp,ymaxp
 	common/test/xtmin, xtmax, ytmin, ytmax
       integer*4 npt,ido
-      real*4 xpt(npt),ypt(npt),dxm(npt),dxp(npt),dym(npt),dyp(npt)
-      real*4 x,y,xlt,xrt,ybt,ytp
+      real xpt(*),ypt(*),dxm(*),dxp(*),dym(*),dyp(*)
+      real x,y,xlt,xrt,ybt,ytp
 
       do 1002 ido = 1, npt
         x = xpt(ido)
@@ -348,8 +391,8 @@ c  filled error box
       common/limits/xminp,xmaxp,yminp,ymaxp
 	common/test/xtmin, xtmax, ytmin, ytmax
       integer*4 npt,ido
-      real*4 xpt(npt),ypt(npt),dxm(npt),dxp(npt),dym(npt),dyp(npt)
-      real*4 x,y,xlt,xrt,ybt,ytp
+      real xpt(*),ypt(*),dxm(*),dxp(*),dym(*),dyp(*)
+      real x,y,xlt,xrt,ybt,ytp
 
       do 1002 ido = 1, npt
         x = xpt(ido)
@@ -385,8 +428,8 @@ c  filled error box
       common/limits/xminp,xmaxp,yminp,ymaxp
 	common/test/xtmin, xtmax, ytmin, ytmax
       integer*4 npt,ido
-      real*4 xpt(npt),ypt(npt),dxm(npt),dxp(npt),dym(npt),dyp(npt)
-      real*4 x,y,xlt,xrt,ybt,ytp
+      real xpt(*),ypt(*),dxm(*),dxp(*),dym(*),dyp(*)
+      real x,y,xlt,xrt,ybt,ytp
 
       do 1002 ido = 1, npt
         x = xpt(ido)
@@ -422,15 +465,15 @@ c  filled error box
 1002  continue
       end
          
-      subroutine smooth(y,npts,ntime,msize)
+      subroutine smooth(y,npts,ntime)
 	include 'robcom'
 c smooth data by convolving with triangle, end points treated
 c slightly differently
-      real y(msize)
+      real y(*)
                    
       if(npts.le.2)then
          write(tstring,'(a, i, a)')
-     +       '***There are only ',NPTS,'in the data set***'
+     +       '***There are only ',NPTS,' points in the data set***'
 	call totext(tstring)
         call xtext('I will not smooth that few data points')
         return
@@ -455,10 +498,9 @@ c rebin data
         subroutine rebin(x,y,delx,dely,npts)
 	include 'robcom'
         real x(*),y(*),delx(*),dely(*)
-c read the rebinning fact0r
+c read the rebinning factor
         call sprompt('Give rebinning factor')
-        call getit(ainbuf,  1)
-        call dcode(ainbuf,ain1,ainfix,aingrd,k)
+        call getinum(1, ain1, ainfix, aingrd, k, 1)
 	fn = ain1(1)
         n=nint(fn)
         i=1
@@ -484,11 +526,93 @@ c read the rebinning fact0r
                   
                   
         end
+
+c 1 D "differentation of data FWIW
+        subroutine differ(x,y,delx,dely,npts)
+	include 'robcom'
+        real x(*),y(*),delx(*),dely(*)
+
+
+	do 10 i = 1, npts-1
+
+	y(i) = y(i) - y(i+1)
+	y(i) = y(i)/(x(i+1) - x(i))
+
+10	continue
+
+	npts = npts - 1
+
+	end
+	
            
+
+c bin the data
+c assume 'time' (or whatever) is in y array
+c (default for single value data)
+	subroutine binup(x, y, delx, dely, wk2)
+	include 'robcom'
+	include 'msizcom'
+        real x(*), y(*), delx(*), dely(*), wk2(*)
+
+	real ttime, tstart, tstep, deltax
+	integer i, ibin, nptsout
+
+	call sprompt('Give start and step for binning')
+	call getinum(1,ain1,ainfix,aingrd,k,2)
+	tstart = ain1(1)
+	tstep = ain1(2)
+
+
+	nptsout = (y(npts)-tstart)/tstep
+
+	if(nptsout .gt. msiz)then
+		call xtext('Need to increase array sizes')
+		write(tstring, *)nptsout, 'points needed'
+		call xtext(tstring)
+		write(tstring,*)'Present array size = ',msiz
+		write(tstring, *)'array size is currently ',msiz
+		return
+	end if
+
+c zero wk2
+	do 10 i = 1, msiz
+		wk2(i) = 0.0
+10	continue
+
+
+	do 50 i = 1, npts
+
+		ttime = y(i) - tstart
+		ibin = int(ttime/tstep)
+		print*,'ibin = ',ibin
+		if(ibin .lt. 0 .or. ibin .gt. msiz)then
+		  call xtext('Funny bin value')
+		  write(tstring, *)'ibin = ',ibin,' y = ',y(i)
+		else
+		   wk2(ibin) = wk2(ibin) + 1.0
+		end if
+50	continue
+
+	deltax = tstep/2.
+
+	do 100 i = 1, nptsout
+	  x(i) = tstart + (i - 1)*tstep
+	  x(i) = x(i) + deltax
+	  y(i) = wk2(i)
+	  delx(i) = deltax
+	  dely(i) = sqrt(y(i))
+100	continue
+
+	npts = nptsout
+
+	end		
+
            
       subroutine linfit(x,y,sigmay,npts,mode,a,sigmaa,b,sigmab,r)
 	include 'robcom'
       real x(*),y(*),sigmay(*)
+
+c	print*,'mode =', mode
                               
 c check whether a weighted fit was requested for a data set
 c without errors
@@ -502,23 +626,16 @@ c doo the real work
                                                       
 C PRINT RESULTS
       call xtext('Results of straight line fit are:')
-      write(tstring,*)'Gradient= '
-	call addrealt(b)
-	call addtextt(' +/- ')
-	call addrealt(sigmab)
-	call totext(tstring)
-      write(tstring,*)'Intercept= '
-		call addrealt(a)
-		call addtextt(' +/- ')
-		call addrealt(sigmaa)
-	call totext(tstring)
-      write(tstring,*)'Linear correlation coefficient='
-		call addrealt(r)
-	call totext(tstring)
       write(ifil8,*)'Results of straight line fit are:'
-      write(ifil8,*)'Gradient= ',b,' +/- ',sigmab
-      write(ifil8,*)'Intercept= ',a,' +/- ',sigmaa
-      write(ifil8,*)'Linear correlation coefficient=',r
+      write(tstring,*)'Gradient = ',b,'+/-',sigmab
+      call totext(tstring)
+      call write8(tstring)
+      write(tstring,*)'Intercept = ',a,' +/- ',sigmaa
+      call totext(tstring)
+      call write8(tstring)
+      write(tstring,*)'Linear correlation coefficient =',r
+      call totext(tstring)
+      call write8(tstring)
 c for small numbers of points evaluate significance of
 c correlation coefficient (only for small numbers to avoid
 c the bevington routine crashing
@@ -527,13 +644,11 @@ c the bevington routine crashing
         call xtext('Probability of getting this level of correlation')
         call xtext('or greater')
         write(tstring,*)
-     +      'with an uncorrelated data set= '
-		call addrealt(PROB*100.)
-		call addtextt('%')
+     +      'with an uncorrelated data set = ',PROB*100.,'%'
 	call totext(tstring)
         write(ifil8,*)'Probability of getting this level of correlation'
         write(ifil8,*)'or greater'
-        write(ifil8,*)'with an uncorrelated data set= ',PROB*100.,'%'
+        write(ifil8,*)'with an uncorrelated data set = ',PROB*100.,'%'
        end if
 	end
     
@@ -583,6 +698,9 @@ c calculate coefficients and standard deviations
 68    sigmab=dsqrt(varnce*sum/delta)
 71    r=(sum*sumxy-sumx*sumy)/
      +dsqrt(delta*(sum*sumy2-sumy*sumy))
+
+c	print*,'delta = ',delta, 'varnce = ',varnce
+c	print*,'sum = ',sum, 'sumx2 = ',sumx2
       end
          
 c significance of correlation coefficient
@@ -675,7 +793,35 @@ c plot fitted function as a dashed line
 	call nodash()
 	end
     
-    
+c write out the value of the fitted function for a specified
+c value of the independent variable
+	subroutine predict(xval)
+	real xval(*)
+	include 'robcom'
+	include 'fitcom'
+        common/robmod/bpoly,norder,fixpar,ifpar,nmodel,mtype
+        common/roba/a,sigmaa
+        common/robspm/ismod
+        real fixpar(maxpar)
+        integer ifpar(maxpar)
+        character*15 mtype(maxmod)
+        real a(maxpar), sigmaa(maxpar)
+        real bpoly(10)
+
+      if(nmodel .eq. 0)then
+        call xtext('CURFIT HAS NOT YET BEEN CALLED, NO PREDICTION MADE')
+        return
+      end if
+      yval = functn(xval, 1, a)
+      write(tstring, *)'For an "x" value of ',xval(1)
+      call totext(tstring)
+      call write8(tstring)
+      write(tstring, *)'the fit predicts a "y" value of ',yval
+      call totext(tstring)
+      call write8(tstring)
+
+	end
+  
     
 c plot results of general function fit
       subroutine pltcur(nstep)
@@ -771,10 +917,10 @@ c routine after error checks etc.
       end if
             
 c check for a colon in the middle
-       do 6 i=1,60
+       do 6 i=1,119
          if(ainbuf(i:i).eq.':')then
            read(ainbuf(1:i-1),*, err = 900)istart
-           read(ainbuf(i+1:60),*, err = 900)iend
+           read(ainbuf(i+1:200),*, err = 900)iend
            goto 5
          end if
 6      continue
@@ -991,15 +1137,24 @@ c fold data and put into bins
 
 c check values are not totally stupid
 	if(nbins .le. 1)then
-          write(tstring,*)'Silly number of bins for folding=',nbins
+          write(tstring,*)'SILLY NUMBER OF BINS FOR FOLDING=',nbins
 	  call xtext(tstring)
+	  return
 	end if
 
+c another zero comparison...
+	if(period .eq. 0.0)then
+          write(tstring,*)'SILLY PERIOD FOR FOLDING=', period
+	  call xtext(tstring)
+	  return
+	end if
+
+
 c zero bins
-      do 50 i=1,nbins
-      	bin(i)=0.0d0
-      	berr(i)=0.0d0
-      	dnbin(i)=0.0d0
+      do 50 i = 1, nbins
+      	bin(i) = 0.0d0
+      	berr(i) = 0.0d0
+      	dnbin(i) = 0.0d0
 50    continue
 
 c inverse of period for later use
@@ -1025,8 +1180,13 @@ c modification to allow for time bins bigger than bin size for folding
 c check for fractional bin values at ends
 c          if(ntry.eq.1)goto 79
           if(ntry.eq.1)then
-             bin(index)=bin(index)+dble(y(i))
-             dnbin(index)=dnbin(index)+1.0d0
+	     if(events)then
+	        bin(index)=bin(index) + 1
+                dnbin(index)=dnbin(index)+1.0d0
+             else
+                bin(index)=bin(index)+dble(y(i))
+                dnbin(index)=dnbin(index)+1.0d0
+	     end if
 	     goto 80
 	  end if
 
@@ -1041,18 +1201,25 @@ c          if(ntry.eq.1)goto 79
 80      continue
 100   continue
       do 150 i=1,nbins
-      if(dnbin(i).ne.0.0)bin(i)=bin(i)/dnbin(i)
+c if it's not event data normalize by the number of samples
+      if(.not.(events))then
+		if(dnbin(i).ne.0.0)bin(i)=bin(i)/dnbin(i)
+	end if
 150   continue
 c standard deviation
+c (only for binned data)
+	if(.not.(events))then
       do 200 i=1,npts
         test=2.0*delx(i)*iperiod
         ntry=test*nbins+1
         step=0.0
         if(ntry.gt.1)step=stepdef
-        start=x(i)-delx(i)
+c        start=x(i)-delx(i)
+	start = x(i) - delx(i) - epoch
         do 190 j=1,ntry
           time=(j-1)*step+start
-          time=(time-epoch)*iperiod
+c          time=(time-epoch)*iperiod
+	  time = time * iperiod
           inp=int(time)
           time=time-inp
           if(time.lt.0.0)time=time+1.0
@@ -1067,6 +1234,11 @@ c standard deviation
 c standard error
          berr(i)=berr(i)/sqrt((dnbin(i)))
 250   continue
+	else
+	do 255 i = 1, nbins
+		berr(i) = sqrt(bin(i))
+255	continue
+	end if
               
 c get chi square for constancy of folded curve
 c only use bins with a least four points in them
@@ -1117,12 +1289,16 @@ c find best period by folding data into bins
         subroutine pfind(x,y,xe,ye,nptsf,bp,nbins)
 	include 'robcom'
         real x(*),y(*),xe(*),ye(*)
+	real epoch
+c epoch is fixed
+	epoch = 0
+
         call sprompt('Give start, stop, step and no. of bins')
-        call getit(ainbuf,  1)
-        call dcode(ainbuf,ain1,ainfix,aingrd,k)
+        call getinum(1, ain1, ainfix, aingrd, k, 4)
 	start = ain1(1)
 	stop = ain1(2)
-	fnbin = ain1(3)
+	step = ain1(3)
+	fnbin = ain1(4)
         nbins=nint(fnbin)
         itimes=nint((stop-start)/step)
         period=start
@@ -1153,6 +1329,9 @@ c and plot the results
         real x(*),y(*),xe(*),ye(*)
 	real wk2(*)
 	logical sizeok
+	real epoch
+
+	epoch = 0
 
 
      
@@ -1248,7 +1427,7 @@ c draw line at chi**2=1 and label
         call ctext(start,-cmax/15.,tstring)
         write(tstring,*)'Best period = ',bp
         call txtm(start,cpmax,tstring,nnl(tstring))
-	write(tstring, *)'Step = ', step
+	write(tstring, *)'Step = ', step, ' Bins = ',nbins
 	call style(stop, cpmax, tstring, nnl(tstring), 1)
 	write(tstring, *)'Largest value of chisqr= ',cmax
 	call xtext(tstring)
@@ -1774,13 +1953,15 @@ c fit polynomial to the data
 c from bevington p.141
       subroutine polfit(x,y,sigmay,npts,nterms,mode,a,chisqr)
 	include 'robcom'
-      double precision sumx(19),sumy(10),xterm,yterm,array(10,10),chisq
-      real x(*),y(*),sigmay(*),a(*)
+      double precision sumx(19), sumy(10), xterm, yterm
+      double precision array(10, 10), chisq
+      real x(*), y(*), sigmay(*), a(*), chisqr, free
 	double precision determ
-	double precision xi, yi, weight
+	double precision xi, yi, weight, delta, term
+	integer nterms, npts, mode, n, i, j, k, nmax, l
 c check whether a weighted fit was requested for a data set
 c without errors
-	if(mode.eq.1.and.sigmay(1).eq.0.0)then
+	if(mode .eq. 1 .and. sigmay(1) .eq. 0.0)then
 		mode = 0
 		call xtext('*Not* weighting by errors')
 		call xtext('First error value zero')
@@ -1790,93 +1971,112 @@ c without errors
 	chisq = 0.0d0
               
 c accumulate weighted sums
-11    nmax=2*nterms-1
-      do 13 n=1,nmax
-13    sumx(n)=0.0d0
-      do 15 j=1,nterms
-15    sumy(j)=0.0d0
-      chisq=0.0d0
-21    do 50 i=1,npts
-      xi=x(i)
-      yi=y(i)
-31    if(mode)32,37,39
-32    if(yi)35,37,33
-33    weight=1./yi
+11    nmax = 2*nterms-1
+      do 13 n = 1, nmax
+13    sumx(n) = 0.0d0
+      do 15 j = 1, nterms
+15    sumy(j) = 0.0d0
+      chisq = 0.0d0
+21    do 50 i = 1, npts
+      xi = x(i)
+      yi = y(i)
+31    if(mode)32, 37, 39
+32    if(yi)35, 37, 33
+33    weight = 1.0d0/yi
       goto 41
-35    weight=1./(-yi)
+35    weight = 1.0d0/(-yi)
       goto 41
-37    weight=1.
+37    weight = 1.0d0
       goto 41
-39    weight=1./(sigmay(i)*sigmay(i))
-41    xterm=weight
-      do 44 n=1,nmax
-      sumx(n)=sumx(n)+xterm
-44    xterm=xterm*xi
-45    yterm=weight*yi
-      do 48 n=1,nterms
-      sumy(n)=sumy(n)+yterm
-48    yterm=yterm*xi
-49    chisq=chisq+weight*yi*yi
+39    weight=1.0d0/dble(sigmay(i)*sigmay(i))
+41    xterm = weight
+      do 44 n = 1, nmax
+      sumx(n) = sumx(n) + xterm
+44    xterm = xterm*xi
+45    yterm = weight*yi
+      do 48 n = 1, nterms
+      sumy(n) = sumy(n) + yterm
+48    yterm = yterm*xi
+      term = weight*yi
+      term = term*yi
+49    chisq = chisq + term
 50    continue
 c construct matrices and calculate coefficients
-51    do 54 j=1,nterms
-      do 54 k=1,nterms
-      n=j+k-1
-54    array(j,k)=sumx(n)
-      delta=determ(array,nterms)
+51    do 54 j = 1, nterms
+      do 54 k = 1, nterms
+      n = j + k - 1
+54    array(j, k) = sumx(n)
+      delta = determ(array, nterms)
       if(delta)61,57,61
-57    chisq=0.0
-      do 59 j=1,nterms
-59    a(j)=0.0
+57    chisq = 0.0
+      do 59 j = 1, nterms
+59    a(j) = 0.0
 	call xtext('WARNING')
 	call xtext('Polfit failed. Maybe too high an order?')
       goto 80
-61    do 70 l=1,nterms
-62    do 66 j=1,nterms
-      do 65 k=1,nterms
-      n=j+k-1
-65    array(j,k)=sumx(n)
-66    array(j,l)=sumy(j)
-70    a(l)=determ(array,nterms)/delta
+61    do 70 l = 1, nterms
+62    do 66 j = 1, nterms
+      do 65 k = 1, nterms
+      n = j + k - 1
+65    array(j, k) = sumx(n)
+66    array(j, l) = sumy(j)
+70    a(l) = determ(array,nterms)/delta
 c calculate chi square
-71    do 75 j=1,nterms
-      chisq=chisq-2.*a(j)*sumy(j)
-      do 75 k=1,nterms
-      n=j+k-1
-75    chisq=chisq+a(j)*a(k)*sumx(n)
-76    free=npts-nterms
-77    chisqr=chisq/free
+71    do 75 j = 1, nterms
+      chisq = chisq - 2.*a(j)*sumy(j)
+      do 75 k = 1, nterms
+      n = j + k - 1
+75    chisq = chisq + dble(a(j))*dble(a(k))*sumx(n)
+76    free = npts - nterms
+77    chisqr = chisq/free
 80    continue
       end
          
-      double precision function determ(array,norder)
+      double precision function determ(array, norder)
 c calculate the determinant of a square matrix
 c from bevington p.294
-      double precision array(10,10),save
-10    determ=1.0d0
-11    do 50 k=1,norder
+      double precision array(10,10), save
+	integer i, j, k, k1, norder
+
+
+10    determ = 1.0d0
+11    do 50 k = 1, norder
 c interchange columns if diagonal element is zero
       if(array(k,k))41,21,41
-21    do 23 j=k,norder
+21    do 23 j = k,norder
       if(array(k,j))31,23,31
 23    continue
-      determ=0.0d0
+      determ = 0.0d0
       goto 60
-31    do 34 i=k,norder
-      save=array(i,j)
-      array(i,j)=array(i,k)
-34    array(i,k)=save
-      determ=-determ
+31    do 34 i = k, norder
+      save = array(i,j)
+      array(i,j) = array(i,k)
+34    array(i,k) = save
+      determ =-determ
 c subtract row k from lower rows to get diagonal matrix
-41    determ=determ*array(k,k)
+41    determ = determ*array(k,k)
 	if(k - norder) 43, 50, 50
-43    k1=k+1
-      do 46 i=k1,norder
-      do 46 j=k1,norder
-46    array(i,j)=array(i,j)-array(i,k)*array(k,j)/array(k,k)
+43    k1 = k+1
+      do 46 i = k1, norder
+      do 46 j = k1, norder
+46    array(i,j) = array(i,j) - array(i,k)*(array(k,j)/array(k,k))
 50    continue
 60    return
       end
+
+
+c kludged call to ps_name_update
+	subroutine pnamup(string)
+	character*(*) string
+	character*125 ostr
+	integer i
+
+	ostr = string
+c add NULL to help C
+	i = MIN(nnl(ostr)+1, len(ostr))
+	ostr(i:i) = char(0)
+	call psnamup(ostr)
+	end
          
          
 	subroutine xtext(string)
@@ -1887,18 +2087,24 @@ c subtract row k from lower rows to get diagonal matrix
 	subroutine totext(string)
 	character*(*) string
 	character*125 ostr
+	integer i
+
 	ostr = string
 c add NULL to help C
-	ostr(nnl(ostr)+1:nnl(ostr)+1) = char(0)
+	i = MIN(nnl(ostr)+1, len(ostr))
+	ostr(i:i) = char(0)
 	call totext2(ostr)
 	end
 
 	subroutine sprompt(string)
 	character*(*) string
 	character*125 ostr
+	integer i
+
 	ostr = string
 c add NULL to help C
-	ostr(nnl(ostr)+1:nnl(ostr)+1) = char(0)
+	i = MIN(nnl(ostr)+1, len(ostr))
+	ostr(i:i) = char(0)
 	call sprompt2(ostr)
 	end
 
@@ -1925,6 +2131,7 @@ c set data colour
 	rgbd(1) = nint(r)
 	rgbd(2) = nint(g)
 	rgbd(3) = nint(b)
+	call cotest(rgbd(1), rgbd(2), rgbd(3), 1)
 	end
 
 
@@ -1938,6 +2145,7 @@ c set colour for everything
 	rgbd(1) = rgb(1)
 	rgbd(2) = rgb(2)
 	rgbd(3) = rgb(3)
+	call cotest(rgb(1), rgb(2), rgb(3), 0)
 	call farkc(rgb(1), rgb(2), rgb(3))
 	end
 
