@@ -129,10 +129,13 @@ c	return
 
 c indentify the nearest point to the specified value
 c (providing its inside the plotting area)
-	subroutine nneigh(xpos, ypos, xminp,yminp,xmaxp,ymaxp)
+	subroutine nneigh(xpos, ypos, xminp,yminp,xmaxp,ymaxp,
+     +			x, y, delx, dely)
 	include 'msizcom'
 	include 'robcom'
 
+	real x(*), y(*), delx(*), dely(*)
+
 
 c only valid if we have data
 	if(npts.le.0)return
@@ -155,9 +158,9 @@ c only valid in plot area
 		end if
 10	continue
 
-	write(tstring,*)'Nearest point is no.',imin
+	write(tstring,*)'Nearest point is number',imin
 	call xtext(tstring)
-	write(tstring,*)'x = ',x(imin), ' y = ',y(imin)
+	write(tstring,*)'x = ',x(imin), ', y = ',y(imin)
 	call xtext(tstring)
 
 	if(delx(imin).ne.0.0)then
@@ -351,6 +354,54 @@ c terminate polygon
 	end
 
 
+c getnum - a call to getit followed by a call to decode
+c part of a slow change to improved handling of input
+
+	subroutine getnum(doit, ain, ifix, igrd, k)
+	integer doit
+	real ain(*)
+	integer ifix(*), igrd(*)
+	integer k
+c a single common block from the robcom file 
+	include 'robcom'
+
+	call getit(ainbuf, doit)
+	call dcode(ainbuf,ain,ifix,igrd,k)
+	end
+
+c requests a specified number of arguments
+	subroutine getinum(doit, ain, ifix, igrd, k, iwant)
+        common/inrob/inter
+	logical inter
+
+	integer doit
+	real ain(*)
+	integer ifix(*), igrd(*)
+	integer iwant
+
+
+	integer k
+c a single common block from the robcom file 
+	include 'robcom'
+
+10	call getit(ainbuf, doit)
+	call dcode(ainbuf,ain,ifix,igrd,k)
+
+	if(k .ne. iwant)then
+		call sprompt('Wrong number of parameters')
+		write(tstring, *)iwant,' needed,', k, ' given'
+		call sprompt(tstring)
+		if(inter)then
+		   goto 10
+		else
+		   call xtext('WARNING: Wrong number of parameters read')
+		end if
+	end if
+		
+
+	end
+
+
 c getit is intended to be the main I/O routine for
 c dialogue interaction within Robot.
 c It will either read from the screen or from a file
@@ -358,7 +409,9 @@ c depending on whether INTER is true or not.
 c This now also automatically writes the input to unit
 c no. 7 too if DOIT is 1.
 	subroutine getit(buff, doit)
-      common/inrob/inter
+        common/inrob/inter
+	common/stdirob/stdin
+	logical stdin
 	logical inter
 	include 'robcom'
 	character*(*) buff
@@ -376,6 +429,9 @@ c somewhere else
 		ibufstr = ibufstr + 1
 c see whether we've now run out of stored data
 		if(ibufstr.ge.maxbuf)HAVEDATA = .FALSE.
+c	else if(stdin)then
+	else if(stdin .and. inter)then
+		read(*, '(a)',err =900,end=901)buff
 	else if(inter) then
 		call gettextxv(buff)
 		call blankit(buff)
@@ -419,7 +475,7 @@ c as required
 	subroutine savdata(buffin)
 c	character buffin(*)
 	character*(*) buffin
-	character*80 buff
+	character*200 buff
 	include 'robcom'
 
 	integer imax, i
@@ -448,7 +504,7 @@ c in a kludgey type of way see whether the instruction has anything
 c after the main instruction - if so assume it's data
 	subroutine chkdat(inst)
 	character*(*)	inst
-	character*80 	buffer
+	character*200 	buffer
 	imax = len(inst)
 	buffer = ' '
 c find where first blank occurs
@@ -736,7 +792,8 @@ c list user variables and their values
 	include 'robcom'
 	
 	do 10 i = 1, nouvar
-		write(tstring, *)uservn(i), '=', uservv(i)
+		write(ainbuf, *)uservn(i)
+		write(tstring, *)ainbuf(1:nnl(ainbuf)),' =',uservv(i)
 		call xtext(tstring)
 10	continue
 	write(tstring, *)'A1 =',auser1
@@ -865,59 +922,20 @@ c increase ulevel for next read to a read to file
 	end
     
 	
-	function nnl(buff)
+	integer function nnl(buff)
 c returns lengths of string which is not spaces or nulls
 	character*(*) buff
 	parameter (ispace = ichar(' '))
 	integer ic
 
 	ilen = len(buff)
-	do 10 i = ilen, 1, -1
-		ic = ichar(buff(i:i))
+	do 10 nnl = ilen, 1, -1
+		ic = ichar(buff(nnl:nnl))
 		if(ic .ne. ispace .and. ic .ne. 0) goto 11
 10	continue
-11	nnl = i
+11	continue
 	end
     
-c adds a real number to the end of a character string which
-c has already had something added to it.
-c Needed to avoid problems if f2c is used to obtain 'C' code
-	subroutine addreal(buff, x)
-	character*(*) buff
-	real x
-	write(buff(nnl(buff)+1:len(buff)), *)x
-	end
-c one for add integer
-	subroutine addint(buff, i)
-	character*(*) buff
-	integer i
-	write(buff(nnl(buff)+1:len(buff)), *)i
-	end
-c a routine for adding a character string
-	subroutine addtext(buff, buff2)
-	character*(*) buff
-	character*(*) buff2
-	buff = buff(1:nnl(buff)+1)//buff2
-	end
-c equivalents of the above hard-wired for writing to "tstring"
-c real routine
-	subroutine addrealt(x)
-	include 'robcom'
-	real x
-	write(tstring(nnl(tstring)+1:len(tstring)), *)x
-	end
-c one for add integer
-	subroutine addintt(i)
-	include 'robcom'
-	integer i
-	write(tstring(nnl(tstring)+1:len(tstring)), *)i
-	end
-c a routine for adding a character string
-	subroutine addtextt(buff2)
-	include 'robcom'
-	character*(*) buff2
-	tstring = tstring(1:nnl(tstring)+1)//buff2
-	end
     
 c close down units 7 and 8
 	subroutine clf()
@@ -965,18 +983,17 @@ c if these fail write to /dev/null
 	integer fpid
 	character*(*) text
 	character*5 pbuf
-	character*100 text2
 	ipid = fpid()
 	write(pbuf, '(i5.5)')ipid
         
-	text2 = '/tmp/'//pbuf(1:nnl(pbuf))//text
+	tstring = '/tmp/'//pbuf(1:nnl(pbuf))//text
                                         
-	open(ifile, file = text2, status = 'new', err = 6)
-	text = text2
+	open(ifile, file = tstring, status = 'new', err = 6)
+	text = tstring(1:nnl(tstring))
 	return
        
 6	print*,'**WARNING***'
-	print*,'COULDN''T OPEN ', text2
+	print*,'COULDN''T OPEN ', tstring
 	print*,'WRITING TO /dev/null'
 	open(ifile, file = '/dev/null', status = 'old', err = 7)
 	return
@@ -1081,7 +1098,7 @@ c sort out x values and errors
      +xfit,yfit,xfite,yfite,nptsf,ifail,istar)
 	include 'robcom'
 c maxcol is maximum number of columns that filred can cope with
-      parameter (maxcol=10)
+      parameter (maxcol=25)
       common/table/index,ftext
       logical ftext
       common/echoc/echo
@@ -1091,9 +1108,11 @@ c maxcol is maximum number of columns that filred can cope with
 c arrays for read routine
       real ain(maxcol)
       integer ifix(maxcol),igrd(maxcol)
-      real x(msiz),y(msiz),dely(msiz),delx(msiz)
+      real x(*),y(*),dely(*),delx(*)
       real xfit(*),yfit(*),xfite(*),yfite(*)
+      integer igood
       ifail=0
+	igood = 1
       open(unit=42,file=pfnam,status='old',err=2)
 	goto 3
 2	continue
@@ -1136,66 +1155,86 @@ c so can't use key words or comments or split variables up
 c with different types of brackets
                                   
       call dcode2(ainbuf,ain,ifix,igrd,k)
+
+c ignore blank lines
+	if(k .eq. 0)goto 5
+
+c if too much data just keep count of how many points
+      if(igood .eq. 0)then 
+		i = i + 1
+		goto 5
+      end if
+
+      if(i .gt. msiz)goto 992
+
       if(k.eq.1)then
         y(i)=ain(1)
         x(i)=real(i)
         delx(i)=0.0
         dely(i)=0.0
-c ignore blank lines
-	else if(k.eq.0)then
-		goto 44
       else
-        if(index(1).gt.0)then
-          x(i)=ain(index(1))
+        if(index(1) .gt. 0)then
+          x(i) = ain(index(1))
         else
-          x(i)=float(i)
+          x(i) = float(i)
         end if
-        if(index(2).gt.0)then
-          y(i)=ain(index(2))
+        if(index(2) .gt. 0)then
+          y(i) = ain(index(2))
         else
-          y(i)=float(i)
+          y(i) = float(i)
         end if
-        if(index(3).gt.0)then
-          dely(i)=ain(index(3))
+        if(index(3) .gt. 0)then
+          dely(i) = ain(index(3))
         else
-          dely(i)=0.0
+          dely(i) = 0.0
         end if
-        if(index(4).gt.0)then
-          delx(i)=ain(index(4))
+        if(index(4) .gt. 0)then
+          delx(i) = ain(index(4))
         else
-          delx(i)=0.0
+          delx(i) = 0.0
         end if
       end if
             
-      xfit(i)=x(i)
-      yfit(i)=y(i)
-      xfite(i)=delx(i)
-      yfite(i)=dely(i)
+      xfit(i) = x(i)
+      yfit(i) = y(i)
+      xfite(i) = delx(i)
+      yfite(i) = dely(i)
       i=i+1
-      if(i.gt.msiz)goto 992
 c zero ain
-      do 44 j=1,maxcol
+      do 44 j=1,k
 44     ain(j)=0.0
       goto 5
 50    continue
               
-      npts = i - 1
-      nptsf = npts
+	if(igood .eq. 1)then
+		npts = i - 1
+      		nptsf = npts
+	end if
       if(echo)then
-		write(tstring,*)'no. of points = ',npts
+		write(tstring,*)'number of points read = ',npts
 		call totext(tstring)
 	end if
       close(unit=42)
+c info message if too much data 
+      if(igood .eq. 0)then
+      		write(tstring, *)'TOTAL SIZE OF INPUT DATA = ',i-1
+		call totext(tstring)
+		call xtext('USE AT LEAST THAT SIZE WITH "ARRAYSIZE"')
+	end if
       return
 C ERROR MESSAGES
 992   call xtext('ERROR: THERE IS TOO MUCH DATA')
       write(tstring,*)'A MAXIMUM OF',MSIZ,' POINTS IS ALLOWED'
 	call totext(tstring)
+	call totext('TO CHANGE THIS USE THE "ARRAYSIZE" COMMAND')
 	npts = i - 1
 	nptsf = npts
-      return
+	igood = 0
+	goto 5
 
 903   call xtext('ERROR OPENING DATA FILE')
+      write(tstring, *)'file: ',pfnam(1:nnl(pfnam)-4)
+      call xtext(tstring)
       IFAIL=1
       return
 993   call xtext('ERROR READING DATA FROM DATA FILE')
@@ -1215,7 +1254,7 @@ C ENTER DATA DIRECTLY INTO PROGRAM
      +title,xlab,ylab,msiz,istar, ilabels)
 	include 'robcom'
 C MAXCOL IS MAXIMUM NUMBER OF COLUMNS THAT FILRED CAN COPE WITH
-      parameter (maxcol=10)
+      parameter (maxcol=25)
       common/table/index,ftext
       logical ftext
       integer index(4)
@@ -1224,8 +1263,12 @@ C ARRAYS FOR READ ROUTINE
       real ain(maxcol)
       integer ifix(maxcol),igrd(maxcol)
       real x(*),y(*),dely(*),delx(*)
+	integer igood
 	character*5 helper(4)
+	common/inrob/inter
+	logical inter
 	data helper/'x','y','yerr ','xerr '/
+	igood = 1
 C Text at top of file?
       IF (FTEXT.and.(ilabels.eq.1))then
 C IDENTIFYING TEXT
@@ -1247,30 +1290,35 @@ c explain which columns are which
 	call sprompt(tstring)
 C READ IN DATA VALUES
       i=istar
-5     call sprompt('Give data, blank line to end')
-        call getit(ainbuf,  1)
-      call dcode(ainbuf,ain,ifix,igrd,k)
+5      if(inter) call sprompt('Give data, blank line to end')
+      call getnum(1,ain,ifix,igrd,k)
       if(k.eq.0)goto 50
+c if too much data just keep count of how many points
+      if(igood .eq. 0)then 
+		i = i + 1
+		goto 5
+      end if
+      if(i.gt.msiz)goto 992
       if(k.eq.1)then
-        y(i)=ain(1)
-        x(i)=real(i)
-        delx(i)=0.0
-        dely(i)=0.0
+        y(i) = ain(1)
+        x(i) = real(i)
+        delx(i) = 0.0
+        dely(i) = 0.0
       else
         if(index(1).gt.0)then
-          x(i)=ain(index(1))
+          x(i) = ain(index(1))
         else
-          x(i)=float(i)
+          x(i) = float(i)
         end if
         if(index(2).gt.0)then
-          y(i)=ain(index(2))
+          y(i) = ain(index(2))
         else
-          y(i)=0.0
+          y(i) = 0.0
         end if
         if(index(3).gt.0)then
           dely(i)=ain(index(3))
         else
-          dely(i)=float(i)
+          dely(i) = 0.0
         end if
         if(index(4).gt.0)then
           delx(i)=ain(index(4))
@@ -1280,25 +1328,30 @@ C READ IN DATA VALUES
       end if
             
       i=i+1
-      if(i.gt.msiz)goto 992
 C ZERO AIN
-      do 44 j=1,maxcol
-44     ain(j)=0.0
+      do 44 j = 1, k
+44     ain(j) = 0.0
       goto 5
 50    continue
               
-      npts=i-1
-      write(tstring,*)'no. of points = ',NPTS
+      if(igood .eq. 1) npts = i - 1
+      write(tstring,*)'number of points read = ',npts
 	call totext(tstring)
+c info message if too much data 
+      if(igood .eq. 0)then
+      		write(tstring, *)'TOTAL SIZE OF INPUT DATA = ',i-1
+		call totext(tstring)
+		call xtext('USE AT LEAST THAT SIZE WITH "ARRAYSIZE"')
+	end if
       return
 C ERROR MESSAGES
 992   write(tstring,'(a, i)')'ERROR: THERE IS TOO MUCH DATA IN UNIT', IFIL
 	call totext(tstring)
-      write(tstring,*)'A MAXIMUM OF'
-	call addintt(MSIZ)
-	call addtextt('POINTS IS ALLOWED')
+      write(tstring,*)'A MAXIMUM OF',MSIZ,' POINTS IS ALLOWED'
 	call totext(tstring)
-      return
+	call totext('TO CHANGE THIS USE THE "ARRAYSIZE" COMMAND')
+	igood = 0
+      goto 5
 c995   write(tstring,'(a, i)')'ERROR READING TEXT HEADER FROM UNIT', IFIL
 c	call totext(tstring)
 c      return
@@ -1315,11 +1368,11 @@ C OUTPUT LABELS AND X,Y POINTS TO SPECIFIED FILE
         open(unit=42, file=pfnam, status = 'new', err = 999)
 c write data to plot file
 c identifying text
-        write(42,'(a)')title
+        write(42,'(a)')title(1:nnl(title))
 c x title
-        write(42,'(a)')xlab
+        write(42,'(a)')xlab(1:nnl(xlab))
 c y title
-        write(42,'(a)')ylab
+        write(42,'(a)')ylab(1:nnl(ylab))
 c write out data values
         do 5 i=1,npts
 	if(x(i).gt.xtmax.or.x(i).lt.xtmin.or.
@@ -1394,8 +1447,8 @@ C FIND START POINT INSIDE DATA LIMITS
        DO 18 I=1,NPTS
 	 xp = x(i)
 	 yp = y(i)
-       if(XP.gt.xtmin.AND.XP.lt.xtmax.AND.YP.gt.Ytmin.
-     +AND.YP.lt.Ytmax)then
+       if(xp.gt.xtmin.and.xp.lt.xtmax.and.yp.gt.ytmin
+     +		.and.yp.lt.ytmax)then
 c        CALL MOVXY(X(I),Y(I))
 	x1 = x(i)
 	y1 = y(i)
@@ -1512,7 +1565,8 @@ c outputlines
 	common/test/xtmin, xtmax, ytmin, ytmax
       real x(*),y(*)
       integer isymb
-                   
+ 
+
       do 10 i=1,npts
         if(x(i).lt.xtmin.or.x(i).gt.xtmax.or.y(i).lt.ytmin.or.y(i).gt.
      +ytmax)goto 10
@@ -1538,7 +1592,7 @@ c plot data as pillars (this is more useful in 3D mode)
 
 	
 c plot data as dashed lines
-      subroutine odlplot(npts,x,y,fmplot,fnplot)
+      subroutine odlplot(npts,x,y)
 	include 'robcom'
       real x(*),y(*)
       common/limits/xminp,xmaxp,yminp,ymaxp
@@ -1571,8 +1625,8 @@ c plot data as dashed lines
 	common/test/xtmin, xtmax, ytmin, ytmax
 	call dodash()
 	do 8 i=1,npts
-	if(x(i).ge.xtmin.and.x(i).le.xtmax.and.y(i).ge.ytmin.
-     +and.y(i).le.ytmax)then
+	if(x(i).ge.xtmin.and.x(i).le.xtmax.and.y(i).ge.ytmin
+     +		.and.y(i).le.ytmax)then
         call movxy(x(i),y(i))
         goto 9
 	end if
@@ -1597,7 +1651,7 @@ c long term ambition is to have hidden line removal
 	subroutine tdplot(npts, x, y, zfalse, 
      +pmode, isymb, symb, sxl, syl, sxh, syh, ntickx, nticky,
      +paxes, ccode, labx, xlab, ylab, zlab,
-     +fiplot, fmplot, fjplot, fnplot, tdfill)
+     +tdfill)
 	include 'robcom'
 	real x(*), y(*), zfalse(*)
 	logical paxes, labx, ccode, tdfill
@@ -1744,7 +1798,7 @@ c complete the box
 	do 55 i = 1, npts
 		if(CCODE)call datcol(zfalse(i), zminp, zscale)
 		call rotate2(x(i), y(i), zfalse(i), x3, y3)
-			call ctit(x3,y3,buffer, nnl(buffer))
+			call ctext(x3,y3,buffer)
 55		continue
 	else if(pmode.eq.'PILLAR')then
 	do 60 i = 1, npts
@@ -1939,6 +1993,79 @@ c used to help show z scale in plot
 		call farkc(icol(1), icol(2), icol(3))
 	end
 
+
+c plot data as a histogram
+	subroutine hplot(npts, x, y)
+	integer npts
+	real x(*), y(*)
+	include 'robcom'
+	common/test/xtmin, xtmax, ytmin, ytmax
+	common/limits/xminp,xmaxp,yminp,ymaxp
+
+	real xp, yp
+C FIND START POINT INSIDE DATA LIMITS
+       do 18 i=1,npts
+	 xp = x(i)
+	 yp = y(i)
+       if(xp .ge. xtmin .and. xp .le. xtmax .and. yp .ge. ytmin
+     +         .and. yp .le. ytmax)then
+        call movxy(x(i),y(i))
+        goto 19
+       end if
+18      continue
+19      J=I
+C EXTEND LINE BACKWARD IF POSSIBLE
+       if(j.gt.1)xp=(x(i)+x(i-1))/2.
+       if(j.eq.1)xp=x(i)-((x(i+1)-x(i))/2.)
+       yp=y(i)
+       if(xp.lt.xtmin)xp=xtmin
+       if(yp.lt.ytmin)yp=ytmin
+       if(xp.gt.xtmax)xp=xtmax
+       if(yp.gt.ytmax)yp=ytmax
+       call linxy(xp,yp)
+
+	xp = x(j)
+	yp = y(j)
+       if(xp .lt. xtmin)xp=xtmin
+       if(yp .lt. ytmin)yp=ytmin
+       if(xp .gt. xtmax)xp=xtmax
+       if(yp .gt. ytmax)yp=ytmax
+       call movxy(xp, yp)
+                            
+       do 110 i=j+1,npts
+         xp=(x(i)+x(i-1))/2.
+         yp=y(i-1)
+         if(yp .gt. ytmax)yp=ytmax
+         if(xp .lt. xtmin)xp=xtmin
+         if(yp .lt. ytmin)yp=ytmin
+         if(xp .gt. xtmax)goto 111
+         call linxy(xp,yp)
+         yp=y(i)
+         if(yp .gt. ytmax)yp=ytmax
+         if(yp .lt. ytmin)yp=ytmin
+         call linxy(xp,yp)
+         xp=x(i)
+         if(xp .gt. xtmax)goto 111
+c for back to front data (e.g. after inverting x)
+	if(xp .lt. xtmin) xp = xtmin
+         call linxy(xp,yp)
+110     continue
+C TERMINATE HISTOGRAM
+       xp=x(npts)+((x(npts)-x(npts-1))/2.)
+       if(xp .gt. xtmax)xp=xtmax
+	if(xp .lt. xtmin)xp = xtmin
+       call linxy(xp,yp)
+        goto 112
+111     continue
+        call linxy(xmaxp,yp)
+112     continue
+
+
+	end
+
+
+
+
 	subroutine barplt(npts, xp, yp, xerr, yerr)
 	real xp(*), yp(*), xerr(*), yerr(*)
 	include 'robcom'
@@ -1959,3 +2086,799 @@ c do a 'bar' plot
 1000	continue
 	end
 
+
+c make sure line width menus set correctly then call
+c actual routine
+
+	subroutine lwid(i)
+	integer i
+	call widchk(i)
+	call lwidth(i)
+
+	end
+
+c make sure font size menu items correctly set up then
+c call the real routine
+
+	subroutine txtcall(size, chsz)
+	real size
+
+	integer isize
+
+	isize = nint(size)
+	
+	call txtchk(isize)
+
+	call txtset(size)
+
+	chsz = nint(size)
+
+	end
+
+c zero a real array
+	subroutine zain(ain, length)
+	real ain(*)
+
+	integer i, length
+
+	do 10 i = 1, length
+		ain(i) = 0.0
+10	continue
+
+	end
+
+c bundles together two routines - FFT and Koji
+c Fourier routine. "itype" determines which one is used
+
+c "slow" Fourier transform
+c rips off an old routine Koji wrote
+c Returns "best period (bp) like
+c folding routines
+
+
+	subroutine power(x, y, ndata, work1, work2, work3, yerr,
+     +		xout, yout, delx, dely, nout,
+     +          start, stop, nf, bp,
+     +          sxh, sxl, syh, syl, itype,
+     +		overwrite, plotft, ifft, iwtft)
+
+	real x(*), y(*), work1(*), work2(*), work3(*)
+	real yerr(*)
+	real xout(*), yout(*), delx(*), dely(*)
+c whether it's an FFT or FT
+	integer itype
+c overwrite determines whether spectrum replaces data
+	logical overwrite
+c plotft says whether we plot the spectrum or not
+	logical plotft
+c What we do if data isn't 2**n points (0 = truncate, 1 = pad,
+c 2 = nearest of previous options
+
+c for adding "photons"
+	double precision addup
+
+
+c for calculating max power/mean power
+	double precision powersum
+
+c 
+	real templ
+
+	integer ifft
+
+
+	include 'robcom'
+	include 'msizcom'
+
+        data twopi/6.2831853/
+	data pi/3.141593/
+
+
+	if(ndata .le. 0)then
+		call xtext('ERROR: Not enough data for transform')
+		write(tstring, *)'Number of points = ', ndata
+		call xtext(tstring)
+		return
+	end if
+
+
+	fmax = 0.0
+	powersum = 0.0d0
+	fbest = fstart
+
+	if(itype .eq. 1)then
+
+	if(nf. gt. msiz)then
+		call xtext('ERROR: TOO MANY FREQUENCIES')
+		write(tstring, *)'Number requested = ', nf
+		call xtext(tstring)
+       		 write(tstring, *)' (Max allowed is ',msiz,')'
+        	call xtext(tstring)
+		call xtext('Use ARRAYSIZE command')
+		return
+	end if
+
+	if(start .ge. stop)then
+		call xtext('ERROR: START VALUE > END VALUE')
+		return
+	end if
+	write(ifil8, *)'Periodogram:'
+	write(ifil8, *)'start =',start,'stop =',stop
+	write(ifil8, *)'Number of frequencies =',nf
+
+c convert periods to frequencies
+	fstart = 1./stop
+	fstop = 1./start
+
+c	print*,'fstart = ',fstart, 'fstop = ', fstop
+
+	fstep = (fstop - fstart)/nf
+
+c	print*,'fstep = ', fstep
+	
+
+	write(tstring, *)'Calculating ', nf,' frequencies...'
+	call xtext(tstring)
+	if(iwtft .eq. 1)then
+		call xtext('(Weighting data by errors)')
+	end if
+
+	fstart = fstart * twopi
+	fstep = fstep * twopi
+
+	call slwft(x, y, work1, work2, ndata, nf, fstart,
+     +			fstep, yerr, iwtft, work3)
+
+	fstart = fstart/twopi
+	fstep = fstep/twopi
+	
+
+c crude normalization
+
+
+	do 10 i = 1, nf
+		work1(i) = work1(i)*work1(i) + work2(i)*work2(i)
+		powersum = powersum + work1(i)
+		if(work1(i) .gt. fmax)then
+			 fmax = work1(i)
+			 fbest = fstart + float(i-1)*fstep
+c and period
+			 bp = 1./fbest
+		end if
+c		print*,work1(i)
+
+10	continue
+
+c	print*,'fmax =',fmax
+
+
+c FFT Part
+	else if (itype .eq. 2)then
+
+
+c initial setting is complete data array
+	n2use = ndata
+
+c put data into work array
+	do 50 i = 1, n2use
+		work1(i) = y(i)
+50		continue
+
+c number of points must be 2**n
+	ftest = n2use
+	ftest = flog2(ftest)
+c this is floating point, do real comparison as integer
+	itest = 2**nint(ftest)
+	if(n2use - itest .ne. 0.0)then
+	   call xtext('WARNING: Number of points not power of 2')
+	   write(tstring, *)'No. of points = ', n2use
+	   call xtext(tstring)
+
+c Do we truncate? Do this if 1) ifft = 0, or nearest, or arrays
+c aren't large enought to pad
+
+c Truncation mode (default) 
+	if(ifft .eq. 0)then
+	   n2use = 2**int(ftest)
+	   write(tstring, *)'truncating to ', n2use
+	   call xtext(tstring)
+
+c Nearest of truncate or pad (in log sense)
+	else if(ifft .eq. 2)then
+	   if(itest .lt. n2use)then
+c we're going to truncate anyway
+	      n2use = 2**int(ftest)
+	      write(tstring, *)'truncating to ', n2use
+	      call xtext(tstring)
+	   else
+	       call fftpad(work1, ndata, n2use, itest, msiz, ftest)
+	   end if
+c Pad mode
+	else if(ifft .eq. 1)then
+	       call fftpad(work1, ndata, n2use, itest, msiz, ftest)
+	else
+c Unknown mode
+	       write(tstring, *)'Unknown FFTMODE of ', ifft
+	       call xtext(tstring)
+	end if
+
+	end if
+
+c crude check whether the data are approximately evenly spaced
+c need at least 3 data points
+
+		if(n2use .ge. 3)then
+
+		   test = x(3) - x(2) - x(2) + x(1)
+		   test = test / (x(2) - x(1))
+		   if(test . gt. 0.01)then
+	call xtext('WARNING: DATA APPEAR TO BE UNEVENLY SPACED!')
+	write(tstring, *)'(Test compared points 1, 2 and 3)'
+	call xtext(tstring)
+		   end if
+c also check start and end times
+		test = (n2use - 1)* (x(2) - x(1)) + x(1)
+c		print*,'test = ',test, 'x(',n2use,') =', x(n2use)
+c		print*, 'x(2) - x(1) =', x(2) - x(1)
+		test = (test - x(n2use)) / (x(2) - x(1))
+		if (abs(test) .gt. 0.01)then
+	call xtext('WARNING: DATA APPEAR TO BE UNEVENLY SPACED!')
+	write(tstring,*)'(Test compared points 1, 2 and ', n2use,')'
+	call xtext(tstring)
+		   end if
+		
+		
+		end if
+
+		nf = n2use/2
+		fstep = 1./(2.*(nf)*(x(2) - x(1)))
+		fstart = 1.00*fstep
+		fstop = (nf )*fstep
+
+c		print*,'fstep = ', fstep
+
+c number of "photons" in spectrum
+		addup = 0.0d0
+		do 400 i = 1, n2use
+		addup = addup + work1(i)
+400		continue
+c		print*,'addup = ',addup
+
+
+		call fft(work1, n2use, 1)
+		work1(1) = 0.0
+		work1(2) = 0.0
+
+c		do 60 i = 0, n2use, 2
+c		temp = work1(i+1)*work1(i+1)
+c		temp = temp + work1(i+2)*work1(i+2)
+c		work1(i/2) = sqrt(temp)/float(nf)
+c
+c60		continue
+
+c Leahy normalization if more than one count
+		if (addup .gt. 1.0)then
+
+		   do 60 i = 0, n2use, 2
+		   temp = work1(i+1)*work1(i+1)
+		   temp = temp + work1(i+2)*work1(i+2)
+		   work1(i/2) = 2.0 * temp /addup
+60		continue
+
+		end if
+
+c Leahy normalization
+		
+
+		do 61 i = 1, nf
+
+		powersum = powersum + work1(i)
+
+		if(work1(i) .gt. fmax)then
+			 fmax = work1(i)
+			 fbest = fstart + float(i-1)*fstep
+c and period
+			 bp = 1./fbest
+c and bin number
+			ibbest = i
+		end if
+61	continue
+
+
+
+	else
+		print*,'Unknown value of itype in Power = ', itype
+	end if
+		
+
+
+	if(plotft)then
+
+        sxh2=(sxh-sxl)*fiplot/fmplot+sxl
+        sxl2=(sxh-sxl)*(fiplot-1.0)/fmplot+sxl
+        syh2=(syh-syl)*fjplot/fnplot+syl
+        syl2=(syh-syl)*(fjplot-1.0)/fnplot+syl
+c allow for window gaps
+        call gap(sxl2, syl2, sxh2, syh2)
+        call limit(sxl2,syl2,sxh2,syh2,fstart,0.0,fstop,fmax*1.2)
+
+        call movxy(fstart, work1(1))
+                                 
+        do 200 i=1,nf
+	  value = fstart + (i-1)*fstep
+          call linxy(value, work1(i))
+200     continue
+        fpmax=1.2*fmax
+        call boxm(fstart,0.0,fstop,fpmax)
+
+	write(tstring, *)'Peak at f = ', fbest, '; Period = ',bp
+	call txtm(fstart, fpmax, tstring, nnl(tstring))
+	write(tstring, *)'Amp. = ',sqrt(fmax)
+
+
+c Also get amplitude based on corrections for binning (frequency
+c dependent terms) and _average_ smearing of a signal
+c (caused by by a signal not corresponding precisely to a Fourier
+c frequency.
+
+	call rtit(fstop, fpmax, tstring)
+
+	write(tstring, *)fstart
+	call ctext(fstart, -fpmax/20., tstring)
+	write(tstring, *)fstop
+	call ctext(fstop, -fpmax/20., tstring)
+	write(tstring, *)'Frequency'
+	call ctext(fstart + (fstop-fstart)/2., -fpmax/10., tstring)
+
+
+c end of plot
+	end if
+
+
+
+	write(tstring, *)
+     + 'Largest amplitude (amp^2) = ',sqrt(fmax), '  (',fmax,')'
+	call xtext(tstring)
+        call write8(tstring)
+
+c power ratio
+	powersum = powersum/dble(nf)
+	write(tstring, *)
+     + 'Ratio of peak power to mean power =', fmax/real(powersum)
+	call xtext(tstring)
+	call write8(tstring)
+
+c for FFT only
+	if(itype .eq. 2)then
+ 	  templ =  sqrt(2.*(fmax -2.0)/addup)
+	  templ = templ*100.
+	  write(tstring, *)'"Leahy" power = ', templ, '%'
+	  call xtext(tstring)
+          call write8(tstring)
+c add non-exact frequency effect
+          templ = templ*sqrt(1./0.773)
+c add binning (frequency dependent) effect
+	  factor = float(ibbest)/(float(n2use)*pi)
+	  factor = sqrt(factor) * sin(factor)
+	  templ = templ*factor
+          write(tstring, *)'With non-Fourier f correction =',templ
+	  call xtext(tstring)
+          call write8(tstring)
+	end if
+	write(tstring, *)'Found at a period of ',bp
+	call xtext(tstring)
+        call write8(tstring)
+	write(tstring, *)'equivalent to a frequency of ',fbest
+	call xtext(tstring)
+        call write8(tstring)
+
+
+
+c if we want to overwrite the data
+	if(overwrite)then
+		nout = nf
+		do 100 i = 1, nf
+			xout(i) = fstart + (i-1)*fstep
+			yout(i) = work1(i)
+			delx(i) = fstep/2.
+			dely(i) = 0.0
+100		continue
+
+	end if
+
+
+
+
+	end
+
+c From a routine by Koji Muaki
+
+	Subroutine slwft( X, Y, C, S, nData, nF, Fmin, dF,
+     +                     yerr, iwtft, work)
+C
+C     Chebyshev recursion formulae used in Paul Murdin's version
+C       accumulate numerical errors, and therefore now replaced
+C       with a better routine taken from the Monro book.
+C
+C                                                       KM 11/7/85
+C
+
+C Yerror array added - experiment with weighting of FT results
+c RC Dec. 1996
+c Idea is to:
+c (1) Calculate weighted mean
+c (2) y -> (y - mean)/error**2
+c (3) to avoid zero power problems subtract off newly calculated
+c mean of data set (bogus?)
+
+C
+	Integer nData, nF
+	Real X( * ), Y( * ), C( * ), S( * ), Fmin, dF
+
+	real yerr(*)
+	integer iwtft
+C
+	Real Z0, dZ, cosK, sinK, dcos, dsin
+	Integer J, K, Interval
+
+
+	real work(*)
+C
+	Do K = 1, nF
+	  S( K ) = 0.0
+	  C( K ) = 0.0
+	End Do
+
+
+c separate normalization into two bits to avoid (?) confusion
+	if(iwtft .eq. 1)then
+		call normftw(work, y, yerr, ndata)
+	else if(iwtft .eq. 0)then
+		call normft(work, y, ndata)
+	else
+		print*,'Unknown weighting = ',iwtft
+		print*,'in slwft'
+	end if
+
+c------
+
+
+c------
+
+c	Print *
+	If( nF .le. 100 ) Then
+	  Interval = 500
+	Else If( nF .le. 200 ) Then
+	  Interval = 500 ! 200
+	Else If( nF .le. 500 ) Then
+	  Interval = 500 ! 100
+	Else
+	  Interval = 500 ! 200
+c were 1000, 500, 200 and 500
+	End If
+	Do J = 1, nData
+	  Z0 = Fmin * X( J )
+	  cosK = COS( Z0 )
+	  sinK = SIN( Z0 )
+	  dZ = dF * X( J )
+	  dcos = COS( dZ )
+	  dsin = SIN( dZ )
+c	  C( 1 ) = C( 1 ) + cosK * Y( J )
+c	  S( 1 ) = S( 1 ) + sinK * Y( J )
+c---------
+c new bit
+	  C( 1 ) = C( 1 ) + cosK * work(j)
+	  S( 1 ) = S( 1 ) + sinK * work(j)
+c end new bit
+c---------
+	  Do K = 2, nF
+	    Call SWING( cosK, sinK, dcos, dsin )
+c	    C( K ) = C( K ) + cosK * Y( J )
+c	    S( K ) = S( K ) + sinK * Y( J )
+c---------
+c new bit
+	    C( K ) = C( K ) + cosK * work(j)
+	    S( K ) = S( K ) + sinK * work(j)
+c end new bit
+c---------
+	  End Do
+c	  If ( MOD( J, Interval ) .eq. 0 ) Then
+c	    Write( *, 110 ) J, nData
+c110	    Format( '+ Processed ', I5, ' out of ', I6, ' records' )
+c	  End If
+	End Do
+c	Write( *, 120 ) nData
+c120	Format( '+ *Now finished processing', I7, ' records.' )
+	Z0 = 2.0 / REAL( nData )
+	Do K = 1, nF
+	  C( K ) = C( K ) * Z0
+	  S( K ) = S( K ) * Z0
+	End Do
+C
+	End
+C
+C
+C
+	Subroutine SWING( cosA, sinA, cosB, sinB )
+C
+
+C
+	Real half, one, one5
+	Parameter( half = 0.5, one = 1.0, one5 = 1.5 )
+C
+	Real cosA, sinA, cosB, sinB
+C
+C       cosA (out) = cos(A+B), sinA (out) = sin(A+B)
+C
+	Real D, Z1, Z2, T
+C
+	D = cosB - one
+	Z1 = cosA + D * cosA - sinA * sinB
+	Z2 = sinA + D * sinA + cosA * sinB
+	T = one5 - half * ( Z1 * Z1 + Z2 * Z2 )
+	cosA = T * Z1
+	sinA = T * Z2
+C
+	End
+
+c Normalize FT - weight by Errors
+	subroutine normftw(work, y, yerr, ndata)
+	real work(*), y(*), yerr(*)
+	integer ndata
+	real mean, factor
+	integer i
+
+	double precision dmean, denoms, denom
+
+c prescale Y values by errors if required (RC)
+c For _all FTs_ we'll subtract off the mean
+	dmean = 0.0d0
+	denoms = 0.0d0
+
+	do 10 i = 1, nData
+	  work(i) = y(i)
+	      if(yerr(i) .ne. 0.0)then
+			denom = 1./(yerr(i)*yerr(i))
+			work(i) = y(i) * denom
+			denoms = denoms + denom
+	      else
+			work(i) = 0.0
+	      end if
+
+	dmean = dmean + work(i)
+
+10	continue
+
+
+	mean = dmean/denoms
+
+
+c For FT we subtract off mean of weighted values
+c instead of weighted mean (I think)
+
+c factor is to keep power spectrum appropriately weighted
+
+	factor = real(nData)/denoms
+
+	do 11 i = 1, nData
+		work(i) = (y(i) - mean)/(yerr(i)*yerr(i))
+		work(i) = factor*work(i)
+11	continue
+
+c check what mean of work array is and subtract off
+c This stage can probably be removed
+
+	dmean = 0.0d0
+	
+	do 12 i = 1, ndata
+		dmean = dmean + work(i)
+12	continue
+
+	mean = dmean/real(ndata)
+
+
+	do 13 i = 1, ndata
+		work(i) = work(i) - mean
+13	continue
+
+	end
+
+c Normalize FT - no weighting by Errors
+	subroutine normft(work, y, ndata)
+	real work(*), y(*)
+	integer ndata
+	double precision dmean
+	real mean
+	integer i
+
+	dmean = 0.0d0
+
+	do 10 i = 1, nData
+	  work(i) = y(i)
+	  dmean = dmean + work(i)
+10	continue
+
+
+	mean = dmean/real(ndata)
+
+c subtract it off
+
+	do 13 i = 1, ndata
+		work(i) = work(i) - mean
+13	continue
+
+
+	end
+
+c FFT routine based on a Numerical Recipes one
+
+	subroutine fft(data, npts, isign)
+      double precision wr,wi,wpr,wpi,wtemp,theta
+      real data(*)
+
+
+	n = npts/2
+
+c	print*,'in fft, n = ', n
+c	print*,'isign = ', isign
+
+      theta = 6.28318530717959d0/2.0d0/dble(n)
+      c1 = 0.5
+      if (isign.eq.1) then
+        c2 = -0.5
+        call fourr(data,n,+1)
+      else
+        c2 = 0.5
+        theta = -theta
+      endif
+      wpr = -2.0d0*dsin(0.5d0*theta)**2
+      wpi = dsin(theta)
+      wr = 1.0d0+wpr
+      wi = wpi
+      n2p3 = 2*n+3
+c      do 11 i = 2,n/2+1
+      do 11 i = 2,n/2
+        i1 = 2*i - 1
+        i2 = i1 + 1
+        i3 = n2p3 - i2
+        i4 = i3 + 1
+        wrs = sngl(wr)
+        wis = sngl(wi)
+        h1r = c1*(data(i1) + data(i3))
+        h1i = c1*(data(i2) - data(i4))
+        h2r = -c2*(data(i2) + data(i4))
+        h2i = c2*(data(i1) - data(i3))
+        data(i1) = h1r + wrs*h2r - wis*h2i
+        data(i2) = h1i + wrs*h2i + wis*h2r
+        data(i3) = h1r - wrs*h2r + wis*h2i
+        data(i4) = -h1i + wrs*h2i + wis*h2r
+        wtemp = wr
+        wr = wr*wpr - wi*wpi +wr
+        wi = wi*wpr + wtemp*wpi +wi
+11    continue
+      if (isign .eq. 1) then
+        h1r = data(1)
+        data(1) = h1r + data(2)
+        data(2) = h1r - data(2)
+      else
+        h1r = data(1)
+        data(1) = c1*(h1r+data(2))
+        data(2) = c1*(h1r-data(2))
+        call fourr(data, n, -1)
+      endif
+      end
+
+
+      subroutine fourr(data,nn,isign)
+      double precision wr,wi,wpr,wpi,wtemp,theta
+      integer nn, isign
+      real data(*)
+
+      n = 2*nn
+      j = 1
+      do 11 i = 1, n, 2
+        if(j .gt. i)then
+          tempr = data(j)
+          tempi = data(j+1)
+          data(j) = data(i)
+          data(j+1) = data(i+1)
+          data(i) = tempr
+          data(i+1) = tempi
+        endif
+        m = n/2
+1       if ((m.ge.2).and.(j.gt.m)) then
+          j = j-m
+          m = m/2
+        go to 1
+        endif
+        j = j+m
+11    continue
+      mmax = 2
+2     if (n .gt. mmax) then
+        istep = 2*mmax
+        theta = 6.28318530717959d0/(isign*mmax)
+        wpr = -2.d0*dsin(0.5d0*theta)**2
+        wpi = dsin(theta)
+        wr = 1.d0
+        wi = 0.d0
+        do 13 m = 1,mmax,2
+          do 12 i = m,n,istep
+            j = i + mmax
+c penny wise, pound foolish
+	    j1 = j + 1
+	    i1 = i + 1
+            tempr = sngl(wr)*data(j)-sngl(wi)*data(j1)
+            tempi = sngl(wr)*data(j1)+sngl(wi)*data(j)
+            data(j) = data(i) - tempr
+            data(j1) = data(i1) - tempi
+            data(i) = data(i) + tempr
+            data(i1) = data(i1) + tempi
+12        continue
+          wtemp = wr
+          wr = wr*wpr-wi*wpi+wr
+          wi = wi*wpr+wtemp*wpi+wi
+13      continue
+        mmax = istep
+      go to 2
+      endif
+
+      end
+
+
+c log base 2 of a number
+	real function flog2(x)
+c inverse log base 10 of 2
+	real log102i
+	real x
+	data log102i/3.321928/
+	flog2 = log10(x)*log102i
+
+	end
+
+
+c pad array with mean of data
+c assumes 
+	subroutine fftpad(work, ndata, n2use, iwant, msiz, ftest)
+	real work(*)
+	integer ndata, n2use, iwant, msiz
+c ftest is log2 data size
+	real ftest
+
+	include 'robcom'
+
+	real*8 ave
+	real ave4
+
+c see if our arrays are big enough
+
+	if(iwant .gt. msiz)then
+		call xtext('Arrays too small to pad')
+		write(tstring, *)'Need a size of at least ',itest
+		call xtext(tstring)
+		n2use = 2**int(ftest)
+		write(tstring, *)'Truncating to ',n2use
+		call xtext(tstring)
+		return
+	end if
+
+c get mean of data
+	ave = 0.0d0
+	do 100 i = 1, ndata
+		ave = ave + work(i)
+100	continue
+
+	ave = ave/dble(ndata)
+	ave4 = ave
+
+	write(tstring, *)'Padding to ',iwant, 'points'
+	call xtext(tstring)
+	write(tstring, *)'With a value of ', ave
+	call xtext(tstring)
+	
+	do 200 i = ndata+1, iwant
+		work(i) = ave
+200	continue
+
+	n2use = iwant
+
+	end
